import { system, world, EquipmentSlot, ItemStack } from '@minecraft/server'

const undead_fragment = new ItemStack('nether:undead_fragment', 1);

world.afterEvents.entityDie.subscribe(event => {
	// Get the damager entity
	const damager = event.damageSource.damagingEntity;
	const deadEntity = event.deadEntity;
	// Necromancy book
	if (damager) {
		// Get the offhand item of the damager
		const offhandItem = getEquipmentSlotItem(damager, EquipmentSlot.Offhand);

		// If the damager is a player
		if (damager.typeId == "minecraft:player" && offhandItem && !event.damageSource.damagingProjectile) {
			// Use the necromancy book
			if (offhandItem.typeId == "nether:necromancy_book" && deadEntity.matches({ families: ["monster"] })
				&& deadEntity.getComponent('minecraft:health').effectiveMax >= 10) {
				const location = deadEntity.location;
				const _world = damager.dimension

				// Spawn the baby skeleton
				const babySkeleton = _world.spawnEntity("nether:baby_skeleton", location);
				babySkeleton.addEffect('resistance', 10, { amplifier: 255, showParticles: false });

				// Spawn particles
				_world.spawnParticle('nether:necromancy_book_particle', location);
				_world.playSound('item.trident.thunder', location, { volume: 0.2, pitch: 4 });
			};
		}

	};

	// If a rotten skeleton has an weakness effect, it will drop an undead fragment
	if (deadEntity && deadEntity.typeId == 'nether:rotten_skeleton') {
		const weakness = deadEntity.getEffect('weakness')
		if (weakness) { deadEntity.dimension.spawnItem(undead_fragment, deadEntity.location); };
	};

});

/**
* @param {import("@minecraft/server").Entity} entity
* @param {EquipmentSlot} slot
*/
function getEquipmentSlotItem(entity, slot) {
	const equipment = entity.getComponent('minecraft:equippable');
	if (!equipment) return undefined;

	const itemStack = equipment.getEquipment(slot);
	return itemStack;
};

world.afterEvents.itemUse.subscribe(event => {
	const holder = event.source;
	const item = event.itemStack;
	if (item.typeId != "nether:infernal_dagger" || holder.getItemCooldown('infernal_dagger') > 0
		|| holder.typeId != "minecraft:player") return;

	const direction = holder.getViewDirection();
	// Push the player
	holder.applyKnockback(direction.x, direction.z, 2.5, 0.4);
	// Add the item cooldown
	holder.startItemCooldown('infernal_dagger', 30);
});

// Particles
world.afterEvents.dataDrivenEntityTrigger.subscribe(event => {
	const entity = event.entity;
	const eventId = event.eventId;

	// Bone collector cast spell
	if (entity.typeId == 'nether:rotten_skeleton') {
		// Resistance effect
		if (eventId == 'nether:rotten_skeleton_resistance') {
			entity.addEffect('resistance', 20 * 20, { amplifier: 1 })
		}
		// Regeneration effect
		else if (eventId == 'nether:rotten_skeleton_regeneration') {
			entity.getComponent('minecraft:health').resetToMaxValue();
			entity.dimension.spawnParticle('nether:necromancy_book_particle', entity.location);
		}
		// Speed effect
		else if (eventId == 'nether:rotten_skeleton_speed') {
			entity.addEffect('speed', 30 * 20, { amplifier: 2 });
		}
	}

	// Baby skeleton despawn particles
	else if (entity.typeId == 'nether:baby_skeleton' && eventId == 'despawn') {
		entity.dimension.spawnParticle('nether:necromancy_book_particle', entity.location);
	};

});
